#!/usr/bin/env python3

import datetime
import logging
import sys
import threading
import time

import RPi.GPIO as gpio
from w1thermsensor import W1ThermSensor

SWITCH_ON_TEMP = 3.0
SWITCH_OFF_TEMP = 5.0

log_level = logging.INFO
LOG_FILE = "/var/log/greenhouse.log"
LOG_FORMAT = "%(asctime)s %(levelname)s %(message)s"

logging.basicConfig(format=LOG_FORMAT, level=log_level, filename=LOG_FILE)
#logging.basicConfig(format=LOG_FORMAT, level=log_level)
log = logging.getLogger('greenhouse')


class EnvDataCollector(threading.Thread):
    def __init__(self, name, update_interval=60):
        super(EnvDataCollector, self).__init__()
        self.name = name
        self.sensor = W1ThermSensor()
        self.update_interval = update_interval
        self.run_condition = True
        self.temperature = self.sensor.get_temperature()
        self.stamp=datetime.datetime.now()

    def run(self):
        next_update = datetime.datetime.now()
        while self.run_condition:
            now = datetime.datetime.now()
            if now >= next_update:
                self.temperature = self.sensor.get_temperature()
                self.stamp = now
                next_update = now + datetime.timedelta(seconds=self.update_interval)
            time.sleep(1)

    def stop(self):
        self.run_condition = False
        self.join()

class Heat(object):
    def __init__(self, pin):
        self.pin = pin
        gpio.setwarnings(False)
        gpio.setmode(gpio.BCM)
        gpio.setup(pin, gpio.OUT)

    def on(self):
        gpio.output(self.pin, 0)

    def off(self):
        gpio.output(self.pin, 1)

def main():
    green = EnvDataCollector("Greenhouse one")
    heat = Heat(26)
    heat.off()
    heat_state = "off"
    green.start()

    try:
        next_update = datetime.datetime.now()
        while True:
            now = datetime.datetime.now()
            if now >= next_update:
                log.info("Temperature: %.1f", float(green.temperature))
                if float(green.temperature) < SWITCH_ON_TEMP and heat_state == "off":
                    heat.on()
                    heat_state = "on"
                    log.info("Heat: switch %s", heat_state)
                elif float(green.temperature) > SWITCH_OFF_TEMP and heat_state == "on":
                    heat.off()
                    heat_state = "off"
                    log.info("Heat: switch %s", heat_state)
                elif heat_state == "on":
                    # Do a power cycle to prevent auto-poweroff
                    heat.off()
                    time.sleep(5)
                    heat.on()
                next_update = now + datetime.timedelta(minutes=5)
            time.sleep(1)
    except KeyboardInterrupt:
        log.info("Shutting down...")
        green.stop()
        log.info("Done")

if __name__ == "__main__":
    sys.exit(main())

